require 'open3'

class Sumodev::Command < Thor
  def self.banner(task, namespace = true, subcommand = false)
    "#{basename} #{task.formatted_usage(self, true, subcommand)}"
  end

  no_commands do
    def run_command_without_output(cmd, in_dir = nil, message = nil, verbose = false)
      if !message.nil?
        say message, :green
      end

      if cmd.kind_of?(Array)
        real_cmd = cmd.join('; ')
      else
        real_cmd = cmd
      end

      if !in_dir.nil?
        real_cmd = "cd #{in_dir}; #{real_cmd}"
      end

      out, error, status = Open3.capture3(real_cmd)

      puts out if verbose
      raise error if status != 0

      return CommandOutput.new out, error, status
    end

    def continue?(message)
      ask colorize("#{message}\nPress a key to continue\n")
    end

    def colorize(message)
      message.gsub(/\*(.*)\*/) { "\033[33m\033[1m#{$1}\033[22m\033[0m" }
    end
  end
end

class CommandOutput
  attr_reader :out, :error, :status

  def initialize(out, error, status)
    @out = out
    @error = error
    @status = status
  end
end
