require 'sumodev/command'
require 'thor/group'

class Sumodev::Commands::Ssh < Thor::Group
  argument :server, :type => :string

  class_option :login

  include Thor::Actions

  Servers = {
    :web1  => {:login => 'root', :host => 'web01.crsolutions.be'},
    :web2  => {:login => 'root', :host => 'web02.crsolutions.be'},
    :mysql => {:login => 'root', :host => 'mysql01.crsolutions.be'},
    :mail  => {:login => 'root', :host => 'mail.crsolutions.be'},
    :mail2 => {:login => 'root', :host => 'mail02.crsolutions.be'},
    :sumodev => {:login => 'sites', :host => 'dev.sumocoders.eu'}
  }

  class << self
    def help(shell)
      super

      shell.say "Possible servers: #{Servers.keys.join(', ')}"
    end
  end

  def check_ssh_key
    if Dir.glob(ENV["HOME"] + "/.ssh/id_[rd]sa").none?
      # No SSH key installed
      if yes?("No SSH key installed! Do you want me to generate one for you?")
        run "ssh-keygen -t rsa -f #{ENV["HOME"]}/.ssh/id_rsa"
      end
    end
  end

  def connect
    server_details = Servers[server.to_sym]

    if server_details
      login = options.fetch(:login, server_details[:login])
      host  = server_details[:host]

      run "ssh #{login}@#{host}"
    else
      say "No such server definition"
    end
  end
  
  private

end
