require 'singleton'

class Sumodev::Config
  include Singleton

  attr_accessor :variables

  class << self
    def get(*args)
      instance.get(*args)
    end

    def true?(*args)
      instance.true?(*args)
    end
  end

  Defaults = {
      'SUMO_TEMP_PATH' => "/tmp/",
      'SUMO_FORWARD_PORTS' => 'true'
  }

  def initialize
    sumofile_path = File.expand_path("~/.sumorc")

    if File.file?(sumofile_path)
      @variables = File.readlines(sumofile_path).inject({}) do |env, line|
        line.chomp!
        next if line.empty?

        line        = line.strip.gsub('export', '')
        name, value = line.split("=", 2)
        env.merge name.strip => value.strip.tr("'\"", '')
      end
    else
      raise("No .sumorc-file found in ~/.sumorc")
    end
  end

  def get(name)
    @variables[name] || Defaults[name] || raise("Can't find setting for #{name} in ~/.sumorc")
  end

  def true?(name)
    ['1', 'true', 'yes', 't', 'y'].member?(get(name).downcase)
  end
end
