require 'sumodev/generator'
require 'sumodev/generators/general/capistrano'

module Sumodev::Generators
  module Fork
    class V3 < Sumodev::Generator
      argument :client, :type => :string
      argument :name, :type => :string

      class_option :fork_repository, :default => "git://github.com/sumocoders/forkcms.git"
      class_option :branch,          :default => 'master'
      class_option :multilanguage,   :default => false, :type => :boolean
      class_option :default_language, :default => 'nl'

      attr_accessor :destination_dir

      def create_root
        self.destination_root = self.destination_dir = Pathname.pwd + client + name
        FileUtils.mkdir_p destination_dir
        FileUtils.chdir   destination_dir
      end

      def checkout_fork
        git :clone    => "#{options[:fork_repository]} ."
        git :checkout => "origin/#{options[:branch]}"
        FileUtils.rm_rf '.git'
      end

      def initialize_repo
        git :init
        git_track 'Initial commit'
      end

      def setup_globals
        git_track 'Setting up globals' do
          copy_dest_file 'library/globals.base.php', 'library/globals.php'
          copy_dest_file 'library/globals_frontend.base.php', 'library/globals_frontend.php'
          copy_dest_file 'library/globals_backend.base.php', 'library/globals_backend.php'

          replace_in_file 'library/globals.php', {
            "'<debug-mode>'"         => 'true',
            "<spoon-debug-email>"    => 'bugs@sumocoders.be',
            "<database-name>"        => name,
            "<database-hostname>"    => 'localhost',
            "<database-port>"        => '3306',
            "<database-username>"    => config[:db][:username],
            "<database-password>"    => config[:db][:password],
            "<site-domain>"          => "#{name}.#{client}.dev",
            "'<site-multilanguage>'" => options[:multilanguage].to_s,
            "<action-group-tag>"     => '@actiongroup',
            "<action-rights-level>"  => 7,
            "<path-www>"             => destination_dir + "default_www",
            "<path-library>"         => destination_dir + "library"
          }

          replace_in_file 'library/globals_frontend.php', {
            '<site-default-language>' => options[:default_language]
          }
        end
      end

      include Sumodev::Generators::General::Capistrano

      private
      # Config
      def config
        # TODO - Fetch this from ~/.sumorc OR ~/.my.cnf
        {
          :db => {
            :username => 'root',
            :password => 'root'
          }
        }
      end
    end
  end
end
