require 'multi_json'

class Sumodev::Sumofile
  attr_accessor :variables

  class << self
    def from_file(path)
      new do |sumofile|
        File.readlines(path).each do |line|
          _, name, value = line.match(/^(.+?)\s+['"]?(.+?)['"]?$/).to_a
          sumofile[name] = value
        end
      end
    rescue Errno::ENOENT
      raise NoSuchFileError
    end

    def current_config
      data = File.read("#{temp_path}/sumofile.json")
      json = MultiJson.load data

      new do |sumofile|
        sumofile.variables = json['sumofile']
      end
    rescue Errno::ENOENT
      raise NoSuchFileError
    end

    def temp_path
      File.expand_path(Sumodev::Config.get('SUMO_TEMP_PATH'));
    end
  end

  def initialize
    @variables = {}

    yield self if block_given?
  end

  def convert_into_json_file
    File.open("#{self.class.temp_path}/sumofile.json", "w") do |file|
      file.write to_json
    end
  end

  def get(name)
    @variables[name]
  end
  alias :[] :get

  def []=(name, value)
    @variables[name] = value
  end

  def to_json
    MultiJson.dump({'sumofile' => @variables})
  end

  class NoSuchFileError < StandardError; end
end
